@@ -7,7 +7,7 @@ import re
 from . import TMPLT_ENV
 from .tool import DrillTool
 from .utilities import read_TO_file
-from ..adripy import get_cdb_location, get_cdb_path, get_full_path
+from ..adripy import get_cdb_location, get_cdb_path, get_full_path, TO_LENGTH_PARAM
 
 class DrillString():
     """
@@ -136,14 +136,51 @@ class DrillString():
                 'Property_File': drill_tool.property_file
             }
     
+    def set_pipe_joints(self, joints, equivalent=False):
+        """Sets the number of joints in the upper most section of
+        drill pipe. Set pipe_type='equivalent' to adjust equivalent
+        upper string joints'
+        
+        Arguments:
+        ----------
+            joints {int}           -- Number of physical drill 
+                                      pipe joints.
+            equivalent {str}       -- False for physical
+                                      string. True for 
+                                      equivalent upper string.
+                                      Default=False
+        
+        Raises:
+        ----------
+            DrillStringError       -- Raised if the drill string
+                                      doesn't have drill pipe 
+                                      of the specified type.
+        """
+        # Check that the pipe type argument was passed correctly
+
+        tool_type = 'equivalent_upper_string' if equivalent else 'drillpipe'
+
+        found = False
+        for tool in self.tools[::-1]:
+            if tool['Type'] == tool_type:
+                # set the number of joints
+                tool['Number_of_Joints'] = joints
+                found = True
+                break
+
+        if not found:          
+            raise DrillStringError(f'There is no {tool_type} in this string!')        
+    
     def get_tool(self, tool_type, instance='first'):
         """Returns a DrillTool object of type tool_type in the 
         DrillString object's tools list.  
         
         Arguments:
+        ----------
             tool_type {str} -- Desired tool type.
         
         Keyword Arguments:
+        ----------
             instance {str or int} -- If 'first', will take the
                                      first instance.  If 'last', 
                                      will take the last instance.
@@ -152,19 +189,67 @@ class DrillString():
                                      (default: {'first'})
         
         Returns:
-            DrillTool -- DrillTool object
+        ----------
+            DrillTool            -- DrillTool object
+        
+        Raises:
+        ----------
+            DrillStringError     -- Raised if a tools of the
+                                    specified type does not exist
+                                    in the drill string.
         """
-
         tools_found = []
         for tool in self.tools:
             if tool['Type']==tool_type:
-                if instance=='first' or len(tools_found)==instance-1:
-                    return tool['DrillTool']
+                if isinstance(instance,str):
+                    if instance=='first':
+                        return tool['DrillTool']
+                    elif instance=='last':
+                        tools_found.append(tool['DrillTool'])
                 else:
-                    tools_found.append(tool['DrillTool'])
+                    if len(tools_found)==instance-1:
+                        return tool['DrillTool']
+                    else:
+                        tools_found.append(tool['DrillTool'])
+
         
+        # Raise an error if no tools were found
+        if tools_found == []:
+            raise DrillStringError(f'No tool of type {tool_type} was found!')
+
         if instance=='last':
             return tools_found[-1]
+        
+    def get_bha_length(self):
+        """
+        Gets the total length of the drill string defined in 
+        string_file NOT including the equivalent upper string and 
+        highest most physical string.
+        
+                        
+        Returns
+        -------
+            {float}:      Cumulative length of the bha    
+        """
+        # Get a list of the drill pipe tools in the string
+        drill_pipe_tools = [tool for tool in self.tools if tool['Type'] == 'drillpipe']
+
+        # Initialize a list of tool lengths
+        tool_lengths = []
+        for tool in self.tools:        
+            # for each tool in the tools list
+            if tool['Type'] != 'equivalent_upper_string' and tool is not drill_pipe_tools[-1]:
+                # If this isn't equivalent upper string or the last drill pipe tool
+                lnth_params = TO_LENGTH_PARAM[tool['Type']]
+                lengths = []
+                for lnth_param in lnth_params:
+                    length = tool['DrillTool'].get_parameter_value(lnth_param)
+                    lengths.append(length)
+                tool_length = sum(lengths)
+                joints = tool['Number_of_Joints']
+                tool_lengths.append(tool_length*joints)        
+        
+        return sum(tool_lengths)
     
     def add_measurement_point(self, distance):
         """
@@ -176,7 +261,7 @@ class DrillString():
         self.parameters['Distance_from_Bit'].append(distance)
         self.parameters['Distance_from_Bit'].sort()
         
-    def write_to_file(self, directory=None, cdb=None, publish=False):
+    def write_to_file(self, directory=None, cdb=None, publish=False, publish_event=False):
         """Creates string file from the DrillString object.
         
         Keyword Arguments:
@@ -208,21 +293,24 @@ class DrillString():
         string_template_1 = TMPLT_ENV.get_template(f'template_1.{self.EXTENSION}')
         string_template_2 = TMPLT_ENV.get_template(f'template_2.{self.EXTENSION}')
         string_template_3 = TMPLT_ENV.get_template(f'template_3.{self.EXTENSION}')
-
         
         if publish is True:
-            # If the string is being published
-
-            # Copy all the tools to the new cdb or directory
+            # If the string is being published, copy all the tools to the new 
+            # cdb or directory
             for tool in self.tools:      
                 tool['DrillTool'].copy_file(cdb=cdb, directory=directory)
                 tool['Property_File'] = tool['DrillTool'].property_file
             
+            # Copy the topdrive file
             self.top_drive['DrillTool'].copy_file(cdb=cdb, directory=directory)
             self.top_drive['Property_File'] = self.top_drive['DrillTool'].property_file
 
-            # Copy the hole and event files to the new cdb
+            # Copy the hole file to the new location
             self._copy_hole_file(cdb=cdb, directory=directory)
+
+        if publish_event is True:
+            # If the string is being published with the event file, copy the
+            # event file to the new location
             self._copy_event_file(cdb=cdb, directory=directory)
 
         with open(filename, 'w') as fid:
@@ -440,4 +528,7 @@ class DrillString():
             
             # Raise a value error if the parameter isn't found.
             if not found:
-                raise ValueError(f'{param} not found!')
\ No newline at end of file
+                raise ValueError(f'{param} not found!')
+
+class DrillStringError(Exception):
+    pass
\ No newline at end of file
